@@ -247,18 +247,18 @@ gail_button_new (GtkWidget *widget)
 static G_CONST_RETURN gchar*
 gail_button_get_name (AtkObject *obj)
 {
+  G_CONST_RETURN gchar* name = NULL;
+
   g_return_val_if_fail (GAIL_IS_BUTTON (obj), NULL);
 
-  if (obj->name != NULL)
-    return obj->name;
-  else
+  name = ATK_OBJECT_CLASS (parent_class)->get_name (obj);
+  if (name == NULL)
     {
       /*
        * Get the text on the label
        */
       GtkWidget *widget;
       GtkWidget *child;
-      G_CONST_RETURN gchar* name = NULL;
 
       widget = GTK_ACCESSIBLE (obj)->widget;
       if (widget == NULL)
@@ -285,8 +285,8 @@ gail_button_get_name (AtkObject *obj)
               name = atk_object_get_name (atk_obj);
             }
         }
-      return name;
     }
+  return name;
 }
 
 static void
@@ -1373,6 +1373,82 @@ gail_button_finalize (GObject            *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static GtkWidget*
+find_label_child (GtkContainer *container,
+                  gint         *index,
+                  gboolean     allow_many)
+{
+  GList *children, *tmp_list;
+  GtkWidget *child;
+ 
+  children = gtk_container_get_children (container);
+
+  child = NULL;
+  for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next) 
+    {
+      if (GTK_IS_LABEL (tmp_list->data))
+        {
+          if (!allow_many)
+            {
+              if (child)
+                {
+                  child = NULL;
+                  break;
+                }
+              child = GTK_WIDGET (tmp_list->data);
+            }
+          else
+            {
+              if (*index == 0)
+                {
+                  child = GTK_WIDGET (tmp_list->data);
+                  break;
+                }
+              *index--;
+	    }
+        }
+       /*
+        * Label for button which are GtkTreeView column headers are in a 
+        * GtkHBox in a GtkAlignment.
+        */
+      else if (GTK_IS_ALIGNMENT (tmp_list->data))
+        {
+          GtkWidget *widget;
+
+          widget = gtk_bin_get_child (GTK_BIN (tmp_list->data));
+          if (GTK_IS_LABEL (widget))
+            {
+              if (!allow_many)
+                {
+                  if (child)
+                    {
+                      child = NULL;
+                      break;
+                    }
+                  child = GTK_WIDGET (tmp_list->data);
+                }
+              else
+                {
+                  if (*index == 0)
+                    {
+	              child = GTK_WIDGET (tmp_list->data);
+                      break;
+                    }
+                  *index--;
+	        }
+	    }
+        }
+      else if (GTK_IS_CONTAINER (tmp_list->data))
+        {
+          child = find_label_child (GTK_CONTAINER (tmp_list->data), index, allow_many);
+          if (child)
+            break;
+        } 
+    }
+  g_list_free (children);
+  return child;
+}
+
 static GtkWidget*
 get_label_from_button (GtkWidget *button,
                        gint      index,
@@ -1389,78 +1465,48 @@ get_label_from_button (GtkWidget *button,
   if (GTK_IS_ALIGNMENT (child))
     child = gtk_bin_get_child (GTK_BIN (child));
 
-  while (GTK_IS_CONTAINER (child))
-    {
-      /*
-       * Child is not a label.  Search for a label in the container's children.
-       */
-      GList *children, *tmp_list;
- 
-      children = gtk_container_get_children (GTK_CONTAINER (child));
-
-      child = NULL;
-      for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next) 
-	{
-	  if (GTK_IS_LABEL (tmp_list->data))
-	    {
-              if (!allow_many)
-                {
-                  if (child)
-                    {
-                      child = NULL;
-                      break;
-                    }
-	          child = GTK_WIDGET (tmp_list->data);
-                }
-              else
-                {
-                  if (n_labels == index)
-                    {
-	              child = GTK_WIDGET (tmp_list->data);
-                      break;
-                    }
-                  n_labels++;
-	        }
-	    }
-           /*
-            * Label for button which are GtkTreeView column headers are in a 
-            * GtkHBox in a GtkAlignment.
-            */
-          if (GTK_IS_ALIGNMENT (tmp_list->data))
-            {
-              GtkWidget *widget;
-
-              widget = gtk_bin_get_child (GTK_BIN (tmp_list->data));
-              if (GTK_IS_LABEL (widget))
-                {
-                  if (!allow_many)
-                    {
-                      if (child)
-                        {
-                          child = NULL;
-                          break;
-                        }
-	              child = GTK_WIDGET (tmp_list->data);
-                    }
-                  else
-                    {
-                      if (n_labels == index)
-                        {
-	                  child = GTK_WIDGET (tmp_list->data);
-                          break;
-                        }
-                      n_labels++;
-	            }
-	        }
-	    }
-	}
-
-      g_list_free (children);
-    }
+  if (GTK_IS_CONTAINER (child))
+    child = find_label_child (GTK_CONTAINER (child), &index, allow_many);
+  else if (!GTK_IS_LABEL (child))
+    child = NULL;
 
   return child;
 }
 
+static void*
+count_labels (GtkContainer *container,
+              gint         *n_labels)
+{
+  GList *children, *tmp_list;
+ 
+  children = gtk_container_get_children (container);
+
+  for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next) 
+    {
+      if (GTK_IS_LABEL (tmp_list->data))
+        {
+          *n_labels++;
+        }
+       /*
+        * Label for button which are GtkTreeView column headers are in a 
+        * GtkHBox in a GtkAlignment.
+        */
+      else if (GTK_IS_ALIGNMENT (tmp_list->data))
+        {
+          GtkWidget *widget;
+
+          widget = gtk_bin_get_child (GTK_BIN (tmp_list->data));
+          if (GTK_IS_LABEL (widget))
+            *n_labels++;
+        }
+      else if (GTK_IS_CONTAINER (tmp_list->data))
+        {
+          count_labels (GTK_CONTAINER (tmp_list->data), n_labels);
+        } 
+    }
+  g_list_free (children);
+}
+
 static gint
 get_n_labels_from_button (GtkWidget *button)
 {
@@ -1472,35 +1518,10 @@ get_n_labels_from_button (GtkWidget *button)
   child = gtk_bin_get_child (GTK_BIN (button));
   if (GTK_IS_ALIGNMENT (child))
     child = gtk_bin_get_child (GTK_BIN (child));
-  while (GTK_IS_CONTAINER (child))
-    {
-      /*
-       * Child is not a label.  Search for a label in the container's children.
-       */
-      GList *children, *tmp_list;
- 
-      children = gtk_container_get_children (GTK_CONTAINER (child));
 
-      child = NULL;
-      for (tmp_list = children; tmp_list != NULL; tmp_list = tmp_list->next) 
-	{
-	  if (GTK_IS_LABEL (tmp_list->data))
-            n_labels++;
+  if (GTK_IS_CONTAINER (child))
+    count_labels (GTK_CONTAINER (child), &n_labels);
 
-           /*
-            * Label for button which are GtkreeView column headers are in a 
-            * GtkHBox in a GtkAlignment.
-            */
-          if (GTK_IS_ALIGNMENT (tmp_list->data))
-            {
-              child = gtk_bin_get_child (GTK_BIN (tmp_list->data));
-	      if (GTK_IS_LABEL (child))
-                n_labels++;
-	    }
-	}
-
-      g_list_free (children);
-    }
   return n_labels;
 }
 
